import json
import logging
from abc import abstractmethod
from cloudsplaining.scan.policy_document import PolicyDocument

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.multi_signature import multi_signature
from checkov.terraform.checks.utils.iam_cloudformation_document_to_policy_converter import \
    convert_cloudformation_conf_to_iam_policy


class BaseCloudsplainingIAMCheck(BaseResourceCheck):
    def __init__(self, name, id):
        super().__init__(name=name, id=id, categories=[CheckCategories.IAM],
            supported_resources=["AWS::IAM::Policy", "AWS::IAM::ManagedPolicy", "AWS::IAM::Group",
            "AWS::IAM::Role", "AWS::IAM::User"])

    def scan_resource_conf(self, conf):
        if 'Properties' in conf.keys():
            props_conf = conf['Properties']
            policies_key = 'Policies'

            # Obtain a list of 1 or more policies regardless of resource schema
            if policies_key in props_conf.keys():
                policy_conf = props_conf[policies_key]
            else:
                policy_conf = [props_conf]

            # Scan all policies
            for policy in policy_conf:
                policy_doc_key = 'PolicyDocument'
                if isinstance(policy, dict) and policy_doc_key in policy.keys():
                    # When using unresolved Cfn functions, policy is an str
                    policy_doc = policy[policy_doc_key]
                    try:
                        converted_policy_doc = convert_cloudformation_conf_to_iam_policy(policy_doc)
                        statement_key = 'Statement'
                        if statement_key in converted_policy_doc:
                            policy_statement = PolicyDocument(converted_policy_doc)
                            violations = self.cloudsplaining_analysis(policy_statement)
                            if violations:
                                logging.debug("detailed cloudsplainging finding: {}",json.dumps(violations))
                                return CheckResult.FAILED
                    except Exception as e:
                        # this might occur with templated iam policies where ARN is not in place or similar
                        logging.debug("could not run cloudsplaining analysis on policy {}", conf)
                        return CheckResult.UNKNOWN
            return CheckResult.PASSED

    @multi_signature()
    @abstractmethod
    def cloudsplaining_analysis(self, policy):
        raise NotImplementedError()



import json
from abc import abstractmethod
from typing import Dict, List, Callable, Optional

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.multi_signature import multi_signature
from checkov.terraform.checks.resource.registry import resource_registry


class BaseResourceCheck(BaseCheck):
    def __init__(self, name: str, id: str, categories: List[CheckCategories], supported_resources: List[str]) -> None:
        super().__init__(
            name=name, id=id, categories=categories, supported_entities=supported_resources, block_type="resource"
        )
        self.supported_resources = supported_resources
        resource_registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, List], entity_type: str) -> CheckResult:
        if conf.get("count") == [0]:
            return CheckResult.UNKNOWN

        self.handle_dynamic_values(conf)
        return self.scan_resource_conf(conf, entity_type)

    @multi_signature()
    @abstractmethod
    def scan_resource_conf(self, conf: Dict[str, List], entity_type: str) -> CheckResult:
        """
        self.evaluated_keys should be set with a JSONPath of the attribute inspected.
        If not relevant it should be set to an empty array so the previous check's value gets overridden in the report.
        """
        raise NotImplementedError()

    @classmethod
    @scan_resource_conf.add_signature(args=["self", "conf"])
    def _scan_resource_conf_self_conf(cls, wrapped: Callable) -> Callable:
        def wrapper(self: "BaseResourceCheck", conf: Dict[str, List], entity_type: Optional[str] = None) -> CheckResult:
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper

    def handle_dynamic_values(self, conf: Dict[str, List]) -> None:
        for dynamic_element in conf.get("dynamic", {}):
            if isinstance(dynamic_element, str):
                try:
                    dynamic_element = json.loads(dynamic_element)
                except Exception:
                    dynamic_element = {}
            for element_name in dynamic_element.keys():
                conf[element_name] = dynamic_element[element_name].get("content", [])



from abc import abstractmethod


class BaseSolver:
    operator = ''

    def __init__(self, solver_type):
        self.solver_type = solver_type

    @abstractmethod
    def get_operation(self, *args, **kwargs):
        raise NotImplementedError()

    @abstractmethod
    def _get_operation(self, *args, **kwargs):
        raise NotImplementedError()

    @abstractmethod
    def run(self, graph_connector):
        raise NotImplementedError()

    @staticmethod
    def resource_type_pred(v, resource_types):
        return len(resource_types) == 0 or v.get('resource_type') in resource_types
